import { browser, by, element } from "protractor";
import { CalculatorPage } from "./calculator.po";
import { ListPage } from "./list.po";
import { Navbar } from "./navbar.po";
import { PreferencesPage } from "./preferences.po"

/**
 * check that fields are empty on creation
 */
describe("ngHyd - Check that examples fields are not empty with 'empty fields on calculator creation' is enabled - ", () => {
    let prefPage: PreferencesPage;
    let navBar: Navbar;
    let calcPage: CalculatorPage;

    beforeEach(async () => {
        prefPage = new PreferencesPage();
        navBar = new Navbar();
        calcPage = new CalculatorPage();

        // enable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await browser.sleep(200);
        await prefPage.enableEvilEmptyFields();
        await browser.sleep(200);
    });

    /**
     * check that a input set is in a given (empty/filled) state
     * @param inputIds id of the inputs to check
     * @param emptys empty/not empty state array
     */
    async function checkFields(inputIds: string[], emptys: boolean[]) {
        let n = 0;
        for (const id of inputIds) {
            const inp = await calcPage.getInputById(id);
            const txt = await inp.getAttribute("value");
            expect(txt === "").toEqual(emptys[n]);
            n++;
        }
    }

    it("when a standard fish ladder calculator is created", async () => {
        // start page
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open 1st example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[0].click();
        await browser.sleep(50);

        // select wall module
        await navBar.openNthCalculator(4);
        await browser.sleep(50);

        // check fields are not empty
        const inputIds = ["Z1", "LB", "PB", "0_L", "0_CdWSL"];
        const emptys = [false, false, false, false, false];
        await checkFields(inputIds, emptys);
    });

    it("calculated parameter initial value when discharge law is modified", async () => {
        // start page
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open 1st example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[0].click();
        await browser.sleep(50);

        // select wall module
        await navBar.openNthCalculator(4);
        await browser.sleep(50);

        // modify 1st structure discharge law
        const dischargeSelect = calcPage.getSelectById("select_loidebit");
        await calcPage.changeSelectValue(dischargeSelect, 1);
        await browser.sleep(200);

        // open initial dialog
        const initDlgButton = element(by.className("param-computed-more"));
        await initDlgButton.click();
        await browser.sleep(200);

        // check input value is not null
        const input = calcPage.getInputById("initval-input");
        const underlyingInput = input.element(by.id("0_h1"));
        const txt = await underlyingInput.getAttribute("value");
        expect(txt === "").toEqual(false);
    });
});

describe("ngHyd - Check that examples work with 'empty fields on calculator creation' enabled - ", () => {
    let prefPage: PreferencesPage;
    let navBar: Navbar;
    let calcPage: CalculatorPage;

    beforeEach(async () => {
        prefPage = new PreferencesPage();
        navBar = new Navbar();
        calcPage = new CalculatorPage();

        // enable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await browser.sleep(200);
        await prefPage.enableEvilEmptyFields();
        await browser.sleep(200);
    });

    it("when calculation is run on a generated fish ladder calculator", async () => {
        // start page
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open 1st example
        const examples = await element.all(by.css("#examples-list .load-example"));
        await examples[0].click();
        await browser.sleep(50);

        // select wall module
        await navBar.openNthCalculator(4);
        await browser.sleep(50);

        // run calculation
        const calcButton = calcPage.getCalculateButton();
        await calcButton.click();
        await browser.sleep(200);

        // click "generate PAB" button
        const genButton = calcPage.getGeneratePabButton();
        await genButton.click();
        await browser.sleep(200);

        // write "6" in basin count input
        const nbBassins = calcPage.getInputById("generatePabNbBassins");
        await nbBassins.sendKeys("6");
        await browser.sleep(50);

        // click "Generate PAB"
        await element(by.css("dialog-generate-pab button#do-generate")).click();
        await browser.sleep(1000);

        // calculate PAB
        const calcButtonPAB = calcPage.getCalculateButton();
        await calcButtonPAB.click();
        await browser.sleep(200);

        // check that result is not empty
        const hasResults = await calcPage.hasResults();
        expect(hasResults).toBe(true);
    });
});
